// filename  ***************  LCD.C ****************************** 
//  LCD Display (HD44780) on PortH, PORTJ for the MC68HC812A4   

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing", Brooks-Cole, copyright (c) 2000,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002,
//    Jonathan W. Valvano 1/30/04

// You may use, edit, run or distribute this file 
//    as long as the above copyright notices remain 
/* 
  size is 1*16
  ground = pin 1    Vss
  power  = pin 2    Vdd   +5V
  ground = pin 3    Vlc   could be tied to 10Kpot
  PJ2    = pin 4    RS    (1 for data, 0 for control/status)
  PJ1    = pin 5    R/W   (1 for read, 0 for write)
  PJ0    = pin 6    E     (enable)
  PH0-7  = pins7-14 DB0-7 (8 bit data)*/

#include <mc68hc812a4.h>     /* derivative information */
#include "LCD.H"

static unsigned short OpenFlag=0;

//---------------------wait---------------------
// time delay
// Input: time in microseconds
// Output: none
void static wait(unsigned short delay){ short TimeLeft,EndT;
  EndT = TCNT+delay;        /* Time (1us cycles) to wait */
  do{
    TimeLeft = (short)TCNT-EndT;
  }
  while((TimeLeft<0)||(TimeLeft>100));   // wait 
  
}
  
//---------------------outCsr---------------------
// sends one command code to the LCD control/status
// Input: command is 8-bit function to execute
// Output: none
static void outCsr(char command){
  PORTH = command;
  PORTJ = 0x00;
  PORTJ = 0x01;  // E goes 0,1
  PORTJ = 0x00;  // E goes 1,0
  wait(90);      // 90 us wait
}

//---------------------LCD_Clear---------------------
// clear the LCD display, send cursor to home
// Input: none
// Output: true if successful
short LCD_Clear(void){
  if(OpenFlag==0){
    return 0;  // not open
  }
  outCsr(0x01);        // Clear Display
  wait(1640);          // 1.64ms wait
  outCsr(0x02);        // Cursor to home
  wait(1640);          // 1.64ms wait
  return 1;		         // success
}
//---------------------LCD_Open---------------------
// initialize the LCD display, called once at beginning
// Input: display determines increment and shift option
//        cursor determines cursor options
//        move determines cursor movement
//        size sets display size
// Output: true if successful
short LCD_Open(char display, char cursor, char move, char size){
  if(OpenFlag){
    return 0;  // already open
  }
  DDRH = 0xFF; DDRJ = 0xFF;
  TSCR  |= 0x80;      // enable
  TMSK2 = 0x23;       // to specify 1us clock
// Be careful, TMSK2 and TSCR maybe set in other rituals
  outCsr(0x04+display); // I/D Increment, S displayshift
  outCsr(0x0C+cursor);  // D=1 displayon, C cursoron/off, B blink on/off
  outCsr(0x14+move);    // S/C cursormove, R/L shiftright
  outCsr(0x30+size);    // DL=1 8bit, N 1/2 line, F 5by7 or 5by10
  OpenFlag = 1;         // device open
  return LCD_Clear();   // clear display
}

//---------------------LCD_OutChar---------------------
// sends one ASCII to the LCD display
// Input: letter is ASCII code
// Output: true if successful
short LCD_OutChar(unsigned char letter){ 
  if(OpenFlag==0){
    return 0;  // not open
  }
  PORTH = letter;
  PORTJ = 0x04;
  PORTJ = 0x05; // E goes 0,1
  PORTJ = 0x04; // E goes 1,0
  wait(90);     // 90 us wait
  return 1;	    // success
}

//---------------------LCD_OutString--------------
// Display String
// Input: pointer to NULL-terminationed ASCII string 
// Output: true if successful
short LCD_OutString(char *pt){ 
  if(OpenFlag==0){
    return 0;  // not open
  }
  while(*pt){
    if(LCD_OutChar((unsigned char)*pt)==0){
      return 0;
    }
    pt++;
  }
  return 1;	  // success
}




